# Movable Type plugin.
#
# Release 0.01 (May 18, 2009) initial version.
#
# Copyright (c) 2004-2009 Yujiro Araki

package MT::Plugin::Email2AddressBook;

use strict;
use base qw(MT::Plugin);

use MT;
use MT::App;

our $VERSION = '0.01';

my $plugin = __PACKAGE__->new({
    id => 'Email2AddressBook',
    key => 'email2addressbook',
    description => '<MT_TRANS phrase="Email2AddressBook.">',
    name => 'Email2AddressBook',
    author_name => 'Yujiro Araki',
    author_link => 'http://www.koikikukan.com/',
    'version' => $VERSION,
    l10n_class => 'Email2AddressBook::L10N',
    system_config_template => 'config.tmpl',
    settings => new MT::PluginSettings([
        ['e2a_title', { Default => '' }],
        ['e2a_account', { Default => '' }],
        ['e2a_passwd', { Default => '' }],
        ['e2a_server', { Default => '' }],
        ['e2a_protocol', { Default => 'pop3' }],
        ['e2a_blog_id', { Default => '' }],
        ['e2a_delete_mail', { Default => '1' }],
    ]),
});

MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        tasks => {
            'Email2AddressBook' => {
                frequency => 3,
                code      => \&_regist_email,
            },
        },
   });
}

use Net::POP3;
use MIME::Base64;
use Email::Address;

sub _regist_email {

  return if(!MT->config->EnableAddressBook);

  my $id = $plugin->get_config_value('e2a_blog_id');
  my $server = $plugin->get_config_value('e2a_server');
  my $account = $plugin->get_config_value('e2a_account');
  my $password = $plugin->get_config_value('e2a_passwd');
  my $protocol = $plugin->get_config_value('e2a_protocol');
  my $title = $plugin->get_config_value('e2a_title') || 'mail_reg';
  my $delete = $plugin->get_config_value('e2a_delete_mail');

  my $mpop  = Net::POP3->new($server) or die "Can't not open server.";
  my $login = (lc($protocol) eq 'apop') ? 'apop' : 'login';
  $mpop->$login($account, $password) or die "Can't not login account.";

  my @popstat = $mpop->popstat();
  my ($from, $subject);

  for my $i (1 .. $popstat[0]) {
    my $line = $mpop->top($i);
    for (@$line){
      chomp;
      if (/^From/) {
        my @addrs = Email::Address->parse($_);
        foreach (@addrs) {
          $from = $_->address;
        }
      } elsif (/^Subject/) {
        s/=\?ISO-2022-JP\?B\?(.*)\?=/MIME::Base64::decode($1)/gei;
        $subject = $_;
        $subject =~ s/^Subject[: ]*//;
      }
    }
    if ($subject eq $title) {
        mail_subscribe($from);
        MT->instance()->log("Subscribe From:" . $from . " Subject:" . $subject);
        if ($delete == 2) {
            $mpop->delete($i);
        }
    }
    if ($delete == 3) {
        $mpop->delete($i);
    }
  }
  $mpop->quit();
}

sub mail_subscribe {
    my $subscr_addr = shift;
    my $blog_id = 1;
    my $app = MT->instance;
#    my $q           = $app->{query};
#    $subscr_addr = lc $q->param('email');
    $subscr_addr =~ s/(^\s+|\s+$)//gs;
    return $app->errtrans("Please enter a valid email address.")
        unless ( $subscr_addr && MT::Util::is_valid_email($subscr_addr) );
#    unless ( $blog_id ) {
#        return $app->errtrans(
#            "Missing required parameter: blog_id. Please consult the user manual to configure notifications."
#        );
#    }

    my $secret           = $app->config->SecretToken;
    my $admin_email_addr = $app->config->EmailAddressMain
        || die "You need to set the EmailAddressMain configuration value "
        . "to your own email address in order to use notifications";
    my $blog = MT::Blog->load( $blog_id )
        || die "No blog found with the given blog_id.";
    my $entry_id = '';
    my $entry;
    if ($entry_id) {
        require MT::Entry;
        $entry = MT::Entry->load(
            {   id      => $entry_id,
                blog_id => $blog->id,
                status  => MT::Entry::RELEASE()
            }
        ) || die "No entry found with the given entry_id.";
    }

    my $redirect_url = $blog->site_url;
    my $site_url     = $blog->site_url;
#    if ( $redirect_url !~ m!\Q$site_url\E! ) {
#        return $app->errtrans(
#            "An invalid redirect parameter was provided. The weblog owner needs to specify a path that matches with the domain of the weblog."
#        );
#    }

    if ( lookup( $blog->id, $subscr_addr ) ) {
        return $app->error(
            $app->translate(
                "The email address '[_1]' is already in the notification list for this weblog.",
                $subscr_addr
            )
        );
    }

    my %head = (
        id      => 'verify_subscribe',
        From    => $admin_email_addr,
        To      => $subscr_addr,
        Subject => $app->translate("Please verify your email to subscribe")
    );
    my $charset = $app->config('MailEncoding')
        || $app->config('PublishCharset');
    $head{'Content-Type'} = qq(text/plain; charset="$charset");

    my @pool = ( 'A' .. 'Z', 'a' .. 'z', '0' .. '9' );
    my $salt = join '', ( map { $pool[ rand @pool ] } 1 .. 2 );
    my $magic = crypt( $secret . $subscr_addr, $salt );
    my $cgipath = $app->config->CGIPath;
    if ( $cgipath =~ m!^/! ) {

        # relative cgipath, prepend blog domain
        my ($blog_domain) = $blog->archive_url =~ m|(.+://[^/]+)|;
        $cgipath = $blog_domain . $cgipath;
    }
    $cgipath .= '/' unless $cgipath =~ m!/$!;
    my $body = MT->build_email(
        'verify-subscribe.tmpl',
        {   script_path  => $cgipath . 'mt-add-notify.cgi',
            blog         => $blog,
            entry        => $entry,
            redirect_url => $redirect_url,
            magic        => $magic,
            email        => $subscr_addr
        }
    );
    use MT::Mail;
    MT::Mail->send( \%head, $body );
}

sub lookup {
    my ( $blog_id, $email ) = @_;

    require MT::Notification;
    my $niter = MT::Notification->load_iter( { blog_id => $blog_id } );
    while ( my $n = $niter->() ) {
        return $n if $n->email eq $email;
    }
    undef;
}

1;
